@@ -53,10 +53,11 @@ LicenseFile=${RW}\\COPYING
 DisableReadyPage=yes
 DisableStartupPrompt=yes
 OutputDir=.
-OutputBaseFilename=SetupR
+OutputBaseFilename=${RW}
 WizardSmallImageFile=R.bmp
 UsePreviousAppDir=no
 ChangesAssociations=yes
+Compression=bzip
 
 [Types]
 Name: "user"; Description: "User installation"
@@ -92,7 +93,6 @@ Root: HKCR; Subkey: "RWorkspace"; ValueType: string; ValueName: ""; ValueData: "
 Root: HKCR; Subkey: "RWorkspace\\DefaultIcon"; ValueType: string; ValueName: ""; ValueData: "{app}\\bin\\RGui.exe,0" 
 Root: HKCR; Subkey: "RWorkspace\\shell\\open\\command"; ValueType: string; ValueName: ""; ValueData: """{app}\\bin\\RGui.exe"" ""%1""" 
 
-
 [Files]
 END
 
