@@ -32,6 +32,7 @@ as.POSIXlt <- function(x, tz = "")
     if(inherits(x, "date") || inherits(x, "dates")) x <- as.POSIXct(x)
     if(is.character(x)) return(fromchar(x))
     if(is.factor(x))	return(fromchar(as.character(x)))
+    if(is.logical(x) && all(is.na(x))) x <- as.POSIXct.default(x)
     if(!inherits(x, "POSIXct"))
 	stop(paste("Don't know how to convert `", deparse(substitute(x)),
 		   "' to class \"POSIXlt\"", sep=""))
@@ -74,6 +75,8 @@ as.POSIXct.default <- function(x, tz = "")
     if(inherits(x, "POSIXct")) return(x)
     if(is.character(x) || is.factor(x))
 	return(as.POSIXct(as.POSIXlt(x), tz))
+    if(is.logical(x) && all(is.na(x)))
+        return(structure(as.numeric(x), class = c("POSIXt", "POSIXct")))
     stop(paste("Don't know how to convert `", deparse(substitute(x)),
 	       "' to class \"POSIXct\"", sep=""))
 }
@@ -533,30 +536,31 @@ cut.POSIXt <-
     x <- as.POSIXct(x)
 
     if (inherits(breaks, "POSIXt")) {
-        breaks <- as.POSIXlt(breaks)
+	breaks <- as.POSIXlt(breaks)
     } else if(is.numeric(breaks) && length(breaks) == 1) {
-        ## specified number of breaks
+	## specified number of breaks
     } else if(is.character(breaks) && length(breaks) == 1) {
-        valid <-
-            pmatch(breaks,
-                   c("secs", "mins", "hours", "days", "weeks",
-                     "months", "years"))
-        if(is.na(valid)) stop("invalid specification of `breaks'")
-        start <- as.POSIXlt(min(x))
-        incr <- 1
-        if(valid > 1) { start$sec <- 0; incr <- 59.99 }
-        if(valid > 2) { start$min <- 0; incr <- 3600 - 1 }
-        if(valid > 3) { start$hour <- 0; incr <- 86400 - 1 }
-        if(valid == 5) {
-            start$mday <- start$mday - start$wday
-            if(start.on.monday)
-                start$mday <- start$mday + ifelse(start$wday > 0, 1, -6)
-            incr <- 7*86400
-        }
-        if(valid == 6) { start$mday <- 1; incr <- 31*86400 }
-        if(valid == 7) { start$mon <- 0; incr <- 366*86400 }
-        breaks <- seq(start, max(x) + incr, breaks)
-        breaks <- breaks[1:(1+max(which(breaks < max(x))))]
+	valid <-
+	    pmatch(breaks,
+		   c("secs", "mins", "hours", "days", "weeks",
+		     "months", "years"))
+	if(is.na(valid)) stop("invalid specification of `breaks'")
+	start <- as.POSIXlt(min(x, na.rm=TRUE))
+	incr <- 1
+	if(valid > 1) { start$sec <- 0; incr <- 59.99 }
+	if(valid > 2) { start$min <- 0; incr <- 3600 - 1 }
+	if(valid > 3) { start$hour <- 0; incr <- 86400 - 1 }
+	if(valid == 5) {
+	    start$mday <- start$mday - start$wday
+	    if(start.on.monday)
+		start$mday <- start$mday + ifelse(start$wday > 0, 1, -6)
+	    incr <- 7*86400
+	}
+	if(valid == 6) { start$mday <- 1; incr <- 31*86400 }
+	if(valid == 7) { start$mon <- 0; incr <- 366*86400 }
+	maxx <- max(x, na.rm = TRUE)
+	breaks <- seq(start, maxx + incr, breaks)
+	breaks <- breaks[1:(1+max(which(breaks < maxx)))]
     } else stop("invalid specification of `breaks'")
     res <- cut(unclass(x), unclass(breaks), labels = labels, right = FALSE)
     if(is.null(labels)) levels(res) <- as.character(breaks[-length(breaks)])
@@ -599,7 +603,7 @@ trunc.POSIXt <- function(x, units=c("secs", "mins", "hours", "days"))
            "secs" = {x$sec <- trunc(x$sec)},
            "mins" = {x$sec <- 0},
            "hours"= {x$sec <- 0; x$min <- 0},
-           "days" = {x$sec <- 0; x$min <- 0; x$hour <- 0}
+           "days" = {x$sec <- 0; x$min <- 0; x$hour <- 0; x$isdst <- -1}
            )
     x
 }
@@ -648,42 +652,48 @@ hist.POSIXt <- function(x, breaks, ..., xlab = deparse(substitute(x)),
     if(!inherits(x, "POSIXt")) stop("wrong method")
     xlab
     x <- as.POSIXct(x)
+    incr <- 1
     ## handle breaks ourselves
     if (inherits(breaks, "POSIXt")) {
         breaks <- as.POSIXct(breaks)
         d <- min(abs(diff(unclass(breaks))))
-        incr <- 1
         if(d > 60) incr <- 60
         if(d > 3600) incr <- 3600
         if(d > 86400) incr <- 86400
         if(d > 86400*7) incr <- 86400*7
         if(d > 86400*28) incr <- 86400*28
         if(d > 86400*366) incr <- 86400*366
-    } else if(is.numeric(breaks) && length(breaks) == 1) {
+        num.br <- FALSE
+    } else {
+        num.br <- is.numeric(breaks) && length(breaks) == 1
+        if(num.br) {
         ## specified number of breaks
-    } else if(is.character(breaks) && length(breaks) == 1) {
-        valid <-
-            pmatch(breaks,
-                   c("secs", "mins", "hours", "days", "weeks",
-                     "months", "years"))
-        if(is.na(valid)) stop("invalid specification of `breaks'")
-        start <- as.POSIXlt(min(x))
-        incr <- 1
-        if(valid > 1) { start$sec <- 0; incr <- 59.99 }
-        if(valid > 2) { start$min <- 0; incr <- 3600 - 1 }
-        if(valid > 3) { start$hour <- 0; incr <- 86400 - 1 }
-        if(valid > 4) { start$isdst <- -1}
-        if(valid == 5) {
-            start$mday <- start$mday - start$wday
-            if(start.on.monday)
-                start$mday <- start$mday + ifelse(start$wday > 0, 1, -6)
-            incr <- 7*86400
+        } else if(is.character(breaks) && length(breaks) == 1) {
+            valid <-
+                pmatch(breaks,
+                       c("secs", "mins", "hours", "days", "weeks",
+                         "months", "years"))
+            if(is.na(valid)) stop("invalid specification of `breaks'")
+            start <- as.POSIXlt(min(x, na.rm = TRUE))
+            incr <- 1
+            if(valid > 1) { start$sec <- 0; incr <- 59.99 }
+            if(valid > 2) { start$min <- 0; incr <- 3600 - 1 }
+            if(valid > 3) { start$hour <- 0; incr <- 86400 - 1 }
+            if(valid > 4) { start$isdst <- -1}
+            if(valid == 5) {
+                start$mday <- start$mday - start$wday
+                if(start.on.monday)
+                    start$mday <- start$mday + ifelse(start$wday > 0, 1, -6)
+                incr <- 7*86400
+            }
+            if(valid == 6) { start$mday <- 1; incr <- 31*86400 }
+            if(valid == 7) { start$mon <- 0; incr <- 366*86400 }
+            maxx <- max(x, na.rm = TRUE)
+            breaks <- seq(start, maxx + incr, breaks)
+            breaks <- breaks[1:(1+max(which(breaks < maxx)))]
         }
-        if(valid == 6) { start$mday <- 1; incr <- 31*86400 }
-        if(valid == 7) { start$mon <- 0; incr <- 366*86400 }
-        breaks <- seq(start, max(x) + incr, breaks)
-        breaks <- breaks[1:(1+max(which(breaks < max(x))))]
-    } else stop("invalid specification of `breaks'")
+        else stop("invalid specification of `breaks'")
+    }
     res <- hist.default(unclass(x), unclass(breaks), plot = FALSE)
     res$equidist <- TRUE # years are of uneven lengths
     res$intensities <- res$intensities*incr
@@ -692,7 +702,9 @@ hist.POSIXt <- function(x, breaks, ..., xlab = deparse(substitute(x)),
         plot(res, xlab = xlab, axes = FALSE, freq = freq, ...)
         if(axes) {
             axis(2, ...)
-            axis.POSIXct(1, at = breaks, format = format)
+            if(num.br)
+                breaks <- c.POSIXct(res$breaks)
+            axis.POSIXct(1, at = breaks,  format = format)
         }
      }
     invisible(res)
