@@ -1,4 +1,4 @@
-options(repositories=c("http://cran.r-project.org/src/contrib"))
+options(repositories=c(CRAN = "http://cran.r-project.org/src/contrib"))
 
 packageStatus <- function(lib.loc = NULL,
                            repositories = getOption("repositories"))
