@@ -6,7 +6,7 @@
 	 family = "Helvetica",
 	 encoding = "default",
 	 pointsize  = 12,
-	 bg	= "white",
+	 bg	= "transparent",
 	 fg	= "black",
 	 onefile    = TRUE,
 	 print.it   = FALSE,
@@ -105,12 +105,15 @@ postscript <- function (file = ifelse(onefile,"Rplots.ps", "Rplot%03d.ps"),
 
     if(is.null(old$command) || old$command == "default")
         old$command <- if(!is.null(cmd <- getOption("printcmd"))) cmd else ""
-    ## handle family separately as length can be 1 or 4
-    if(!missing(family)) old$family <- family
+    ## handle family separately as length can be 1, 4, or 5
+    if(!missing(family)) {
+        if(length(family) == 4) family <- c(family, "sy______.afm")
+        old$family <- family
+    }
     if(is.null(old$encoding) || old$encoding  == "default")
-        old$encoding <- switch(machine(),
-                               "Macintosh" = "MacRoman.enc",
-                               "Win32" = "WinAnsi.enc",
+        old$encoding <- switch(.Platform$OS.type,
+                               "mac" = "MacRoman.enc",
+                               "windows" = "WinAnsi.enc",
                                "ISOLatin1.enc")
     .Internal(PS(file, old$paper, old$family, old$encoding, old$bg, old$fg,
 		 old$width, old$height, old$horizontal, old$pointsize,
@@ -136,9 +139,9 @@ pdf <- function (file = ifelse(onefile, "Rplots.pdf", "Rplot%03d.pdf"),
     old <- check.options(new = new, name.opt = ".PostScript.Options",
 			 reset = FALSE, assign.opt = FALSE)
     if(is.null(old$encoding) || old$encoding  == "default")
-        old$encoding <- switch(machine(),
-                               "Macintosh" = "MacRoman.enc",
-                               "Win32" = "WinAnsi.enc",
+        old$encoding <- switch(.Platform$OS.type,
+                               "mac" = "MacRoman.enc",
+                               "windows" = "WinAnsi.enc",
                                "ISOLatin1.enc")
     .Internal(PDF(file, old$family, old$encoding, old$bg, old$fg,
                   width, height, old$pointsize, old$onefile))
