@@ -1,15 +1,9 @@
 showDefault <-
-  function(object, printTo = stdout(), oldMethods = TRUE)
+  function(object, oldMethods = TRUE)
 {
-      if(identical(printTo, FALSE)) {
-          tmp <- tempfile()
-          con <- file(tmp, "w")
-      }
-      else
-          con <- printTo
-    cl <- data.class(object)
+    cl <- .class1(object)
     if(isClass(cl) && is.na(match(cl, .BasicClasses))) {
-        cat(file = con, "An object of class \"", cl, "\"\n", sep="")
+        cat("An object of class \"", cl, "\"\n", sep="")
         slots <- slotNames(cl)
         if(!is.na(match(".Data", slots))) {
             dataPart <- object@.Data
@@ -21,9 +15,9 @@ showDefault <-
         for(what in slots) {
             if(identical(what, ".Data"))
                 next ## should have been done above
-            cat(file = con, "Slot \"",what, "\":\n", sep="")
+            cat("Slot \"",what, "\":\n", sep="")
             print(slot(object, what))
-            cat(file = con, "\n")
+            cat("\n")
         }
     }
     else {
@@ -34,23 +28,16 @@ showDefault <-
             if(existsFunction(oldMethod))
                 printFun <- getFunction(oldMethod, generic = FALSE)
         }
-        if(identical(printTo, FALSE)) {
-            sink(con)
-            printFun(object)
-            sink()
-        }
-        else
-            printFun(object)
+        printFun(object)
      }
-
-    if(identical(printTo, FALSE)) {
-        close(con)
-        value <- readLines(tmp)
-        unlink(tmp)
-        value
-    }
 }
 
+## temporary definition of show, to become the default method
+## when .InitShowMethods is called
+show <- function(object)
+    showDefault(object, FALSE)
+
+
 printNoClass <- get("print.default", "package:base")
 
 print.default <- function(x, ...) {
@@ -62,16 +49,12 @@ print.default <- function(x, ...) {
 }
 
 .InitShowMethods <- function(envir) {
-    setGeneric("show", function(object)standardGeneric("show"),
-               where = envir)
-    setMethod("show", "ANY",
-              function(object)
-              showDefault(object, oldMethods = FALSE), where = envir)
+    setGeneric("show", where = envir)
     setMethod("show", "MethodDefinition",
               function(object) {
                   cat("Method Definition (Class \"", class(object), "\"):\n\n", sep = "")
                   show(object@.Data)
-                  mm <- methodSignatureMatrix(object)
+                  mm <- .methodSignatureMatrix(object)
                   cat("\nSignatures:\n")
                   print(mm)
               },
@@ -80,11 +63,27 @@ print.default <- function(x, ...) {
               function(object)  {
                   cat("Method Definition (Class \"", class(object), "\"):\n\n", sep = "")
                   show(object@.Data)
-                  mm <- rbind(methodSignatureMatrix(object),
+                  mm <- rbind(.methodSignatureMatrix(object),
                               NextMethod = object@nextMethod@defined)
                   cat("\nSignatures:\n")
                   print(mm)
               },
               where = envir)
-    
+    setMethod("show", "genericFunction",
+              function(object)  {
+                  cat(class(object)," for \"", object@generic,
+                      "\" defined from package \"", object@package,
+                      "\"\n", sep = "")
+                  if(length(object@group) > 0)
+                      cat("  belonging to group(s):",
+                          paste(unlist(object@group), collapse =", "), "\n")
+                  if(length(object@valueClass) > 0)
+                      cat("  defined with value class: \"", object@valueClass,
+                          "\"\n", sep="")
+                  cat("\n")
+                  show(object@.Data)
+                  cat("Methods may be defined for arguments:",
+                      paste(object@signature, collapse=", "), "\n\n")
+              },
+              where = envir)
 }
