@@ -300,24 +300,7 @@ predict.arima0 <-
     } else return(pred)
 }
 
-arima0.diag <- function(object, gof.lag = 10)
-{
-    .Deprecated("tsdiag")
-    ## plot standardized residuals, acf of residuals, Box-Pierce p-values
-    oldpar<- par(mfrow = c(3, 1))
-    on.exit(par(oldpar))
-    rs <- object$resid
-    stdres <- rs/sqrt(object$sigma2)
-    plot(stdres, type = "h", main = "Standardized Residuals", ylab = "")
-    abline(h = 0)
-    acf(object$resid, plot = TRUE, main = "ACF of Residuals")
-    nlag <- gof.lag
-    pval <- numeric(nlag)
-    for(i in 1:nlag) pval[i] <- Box.test(rs, i)$p.value
-    plot(1:nlag, pval, xlab = "lag", ylab = "p value", ylim = c(0,1),
-         main = "p values for Box-Pierce statistic")
-    abline(h = 0.05, lty = 2, col = "blue")
-}
+arima0.diag <- function(...) .Defunct()
 
 tsdiag.Arima <- tsdiag.arima0 <- function(object, gof.lag = 10, ...)
 {
