@@ -98,7 +98,7 @@ spec.pgram <-
             x[, i] <- x[, i] - mean(x[, i]) - sum(x[, i] * t) * t/sumt2
     }
     else if (demean) {
-        x <- sweep(x, 2, apply(x, 2, mean))
+        x <- sweep(x, 2, colMeans(x))
     }
     x <- spec.taper(x, taper)
     ## to correct for tapering: Bloomfield (1976, p. 194)
@@ -182,7 +182,7 @@ plot.spec <-
             stop("coverage probability out of range [0,1)")
         tail <- (1 - coverage)
         df <- spec.obj$df
-        upper.quantile <- 1 - tail * (1 - pchisq(df, df))
+        upper.quantile <- 1 - tail * pchisq(df, df, lower.tail = FALSE)
         lower.quantile <- tail * pchisq(df, df)
         1/(qchisq(c(upper.quantile, lower.quantile), df)/df)
     }
@@ -213,8 +213,7 @@ plot.spec <-
     } else {
         matplot(x$freq, x$spec, xlab = xlab, ylab = ylab, type = type,
                 log = ylog, ...)
-        is.ar <- !is.na(pmatch("AR", x$method))
-        if (ci <= 0 || log == "no" || is.ar) {
+        if (ci <= 0 || !is.numeric(x$df) || log == "no") {
             ## No confidence limits
             ci.text <- ""
         } else {
@@ -242,8 +241,10 @@ plot.spec <-
             }
         }
         if (is.null(main))
-            main <- paste(paste("Series:", x$series), x$method, sep = "\n")
-        if (is.null(sub) && !is.ar)
+            main <- paste(if(!is.null(x$series)) paste("Series:", x$series)
+                          else "from specified model",
+                          x$method, sep = "\n")
+        if (is.null(sub) && is.numeric(x$bandwidth))
              sub <- paste("bandwidth = ", format(x$bandwidth, digits = 3),
                          ci.text, sep="")
         title(main = main, sub = sub)
