@@ -75,7 +75,9 @@
 
 #include "Defn.h"
 
+#if 0
 static SEXP gcall;
+#endif
 
 /* EnlargeVector() takes a vector "x" and changes its length to "newlen".  
    This allows to assign values "past the end" of the vector or list.
@@ -125,7 +127,7 @@ static SEXP EnlargeVector(SEXP x, int newlen)
 	for (i = 0; i < len; i++)
 	    SET_STRING_ELT(newx, i, STRING_ELT(x, i));
 	for (i = len; i < newlen; i++)
-	    SET_STRING_ELT(newx, i, R_BlankString);
+	    SET_STRING_ELT(newx, i, NA_STRING); /* was R_BlankString  < 1.6.0 */
 	break;
     case EXPRSXP:
     case VECSXP:
@@ -152,7 +154,7 @@ static SEXP EnlargeVector(SEXP x, int newlen)
     return newx;
 }
 
-static int SubassignTypeFix(SEXP *x, SEXP *y, int stretch, int level)
+static int SubassignTypeFix(SEXP *x, SEXP *y, int stretch, int level, SEXP call)
 {
     Rboolean redo_which =  (level == 1);
     int which = 100 * TYPEOF(*x) + TYPEOF(*y);
@@ -265,7 +267,7 @@ static int SubassignTypeFix(SEXP *x, SEXP *y, int stretch, int level)
 	break;
 
     default:
-	errorcall(gcall, "incompatible types");
+	errorcall(call, "incompatible types");
 
     }
 
@@ -357,7 +359,7 @@ static SEXP VectorAssign(SEXP call, SEXP x, SEXP s, SEXP y)
     /* Here we make sure that the LHS has */
     /* been coerced into a form which can */
     /* accept elements from the RHS. */
-    which = SubassignTypeFix(&x, &y, stretch, 1);
+    which = SubassignTypeFix(&x, &y, stretch, 1, call);
     /* = 100 * TYPEOF(x) + TYPEOF(y);*/
     ny = length(y);
     nx = length(x);
@@ -617,7 +619,7 @@ static SEXP MatrixAssign(SEXP call, SEXP x, SEXP s, SEXP y)
     if (n > 0 && n % ny)
 	errorcall(call, "number of items to replace is not a multiple of replacement length");
 
-    which = SubassignTypeFix(&x, &y, 0, 1);
+    which = SubassignTypeFix(&x, &y, 0, 1, call);
 
     PROTECT(x);
 
@@ -794,6 +796,22 @@ static SEXP MatrixAssign(SEXP call, SEXP x, SEXP s, SEXP y)
 	    }
 	}
 	break;
+    case 1919: /* vector <- vector */
+
+	for (j = 0; j < ncs; j++) {
+	    jj = INTEGER(sc)[j];
+	    if (jj == NA_INTEGER) continue;
+	    jj = jj - 1;
+	    for (i = 0; i < nrs; i++) {
+		ii = INTEGER(sr)[i];
+		if (ii == NA_INTEGER) continue;
+		ii = ii - 1;
+		ij = ii + jj * nr;
+		SET_VECTOR_ELT(x, ij, VECTOR_ELT(y, k));
+		k = (k + 1) % ny;
+	    }
+	}
+	break;
     default:
 	error("incompatible types in subset assignment");
     }
@@ -853,7 +871,7 @@ static SEXP ArrayAssign(SEXP call, SEXP x, SEXP s, SEXP y)
     /* Here we make sure that the LHS has been coerced into */
     /* a form which can accept elements from the RHS. */
 
-    which = SubassignTypeFix(&x, &y, 0, 1);/* = 100 * TYPEOF(x) + TYPEOF(y);*/
+    which = SubassignTypeFix(&x, &y, 0, 1, call);/* = 100 * TYPEOF(x) + TYPEOF(y);*/
 
     if (ny == 0) {
 	UNPROTECT(1);
@@ -958,6 +976,14 @@ static SEXP ArrayAssign(SEXP call, SEXP x, SEXP s, SEXP y)
 
 	    SET_STRING_ELT(x, ii, STRING_ELT(y, i % ny));
 	    break;
+
+	case 1919: /* vector <- vector */
+
+	    SET_VECTOR_ELT(x, ii, VECTOR_ELT(y, i % ny));
+	    break;
+
+	default:
+	    error("incompatible types in subset assignment");
 	}
 	if (n > 1) {
 	    j = 0;
@@ -1125,14 +1151,20 @@ static SEXP listAssign1(SEXP call, SEXP x, SEXP subs, SEXP y)
 static void SubAssignArgs(SEXP args, SEXP *x, SEXP *s, SEXP *y)
 {
     SEXP p;
-    if (length(args) < 3)
+    if (length(args) < 2)
 	error("SubAssignArgs: invalid number of arguments");
     *x = CAR(args);
-    *s = p = CDR(args);
-    while (CDDR(p) != R_NilValue)
-	p = CDR(p);
-    *y = CADR(p);
-    SETCDR(p, R_NilValue);
+    if(length(args) == 2) {
+	*s = R_NilValue;
+	*y = CADR(args);
+    }
+    else {
+	*s = p = CDR(args);
+	while (CDDR(p) != R_NilValue)
+	    p = CDR(p);
+	*y = CADR(p);
+	SETCDR(p, R_NilValue);
+    }
 }
 
 
@@ -1162,7 +1194,6 @@ SEXP do_subassign_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP subs, x, y;
     int nsubs, oldtype;
 
-    gcall = call;
     PROTECT(args);
 
     /* If there are multiple references to an object we must */
@@ -1204,6 +1235,7 @@ SEXP do_subassign_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     case VECSXP:
 	switch (nsubs) {
 	case 0:
+	    x = VectorAssign(call, x, R_MissingArg, y);
 	    break;
 	case 1:
 	    x = VectorAssign(call, x, CAR(subs), y);
@@ -1286,8 +1318,6 @@ SEXP do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP dims, indx, names, newname, subs, x, y;
     int i, ndims, nsubs, offset, stretch, which;
 
-    gcall = call;
-
     PROTECT(args);
 
     SubAssignArgs(args, &x, &subs, &y);
@@ -1356,7 +1386,7 @@ SEXP do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    UNPROTECT(1);
 	}
 
-	which = SubassignTypeFix(&x, &y, stretch, 2);
+	which = SubassignTypeFix(&x, &y, stretch, 2, call);
 	PROTECT(x);
 
 	switch (which) {
@@ -1525,7 +1555,7 @@ SEXP do_subassign2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 	UNPROTECT(1);
     }
-    else errorcall(gcall, "object is not subsetable");
+    else errorcall(call, "object is not subsetable");
 
     UNPROTECT(1);
     SET_NAMED(x, 0);
@@ -1575,8 +1605,6 @@ SEXP R_subassign3_dflt(SEXP call, SEXP x, SEXP nlist, SEXP val)
     SEXP t;
     PROTECT_INDEX pvalidx;
 
-    gcall = call;
-
     PROTECT(x);
     PROTECT_WITH_INDEX(val, &pvalidx);
 
