@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Langage for Statistical Data Analysis
- *  Copyright (C) 1998-2000   Lyndon Drake
+ *  Copyright (C) 1998-2002   Lyndon Drake
  *                            and the R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,8 +18,18 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 #include "Defn.h"
 #include "Fileio.h"
+#ifdef HAVE_FCNTL_H
+# include <fcntl.h> /* for open and constants */
+#endif
+#ifdef HAVE_SYS_STAT_H
+# include <sys/stat.h>
+#endif
 
 #include "terminal.h"
 #include "terminal-prefs.h"
@@ -246,6 +256,8 @@ void pager_set_style()
   */
 }
 
+#define BUFSIZE  2048
+
 int Rgnome_ShowFiles(int nfile, char **file, char **title, char *wtitle,
 		     Rboolean del, char *pager) 
 {
@@ -257,8 +269,7 @@ int Rgnome_ShowFiles(int nfile, char **file, char **title, char *wtitle,
   GdkFont *titlefont, *emfont;
   GdkColor textcolor, bgcolor;
 
-  const gint bufsize = 2048;
-  gchar buf[bufsize];
+  gchar buf[BUFSIZE];
   gint i;
   gint fd, readlen;
   gchar *j, *k;
@@ -317,7 +328,7 @@ int Rgnome_ShowFiles(int nfile, char **file, char **title, char *wtitle,
 
   for(i = 0; i < nfile; i++) {
     if((title[i] != NULL) && (*title[i] != '\0')) {
-      g_snprintf(buf, bufsize, "%s\n\n", title[i]);
+      g_snprintf(buf, BUFSIZE, "%s\n\n", title[i]);
       gtk_text_insert(GTK_TEXT(pager_data->text), 
 		      titlefont,
 		      &textcolor, 
@@ -326,7 +337,7 @@ int Rgnome_ShowFiles(int nfile, char **file, char **title, char *wtitle,
     }
     if((fd = open(file[i], O_RDONLY, "")) != -1) {
       do {
-	readlen = read(fd, buf, bufsize);
+	readlen = read(fd, buf, BUFSIZE);
 
 	emmode = FALSE;
 	modestart = buf;
@@ -360,10 +371,10 @@ int Rgnome_ShowFiles(int nfile, char **file, char **title, char *wtitle,
 
 	gtk_text_insert(GTK_TEXT(pager_data->text), NULL, NULL, NULL,
 			modestart, k - modestart);
-      } while(readlen == bufsize);
+      } while(readlen == BUFSIZE);
     }
     else {
-      g_snprintf(buf, bufsize, "NO FILE %s\n\n", file[i]);
+      g_snprintf(buf, BUFSIZE, "NO FILE %s\n\n", file[i]);
       gtk_text_insert(GTK_TEXT(pager_data->text), NULL, NULL, NULL,
 		      buf, strlen(buf));
     }
