@@ -21,8 +21,6 @@ int  Rstd_ChooseFile(int new, char *buf, int len);
 void Rstd_loadhistory(SEXP call, SEXP op, SEXP args, SEXP env);
 void Rstd_savehistory(SEXP call, SEXP op, SEXP args, SEXP env);
 
-char *Runix_tmpnam(char * prefix);
-
 void Rgnome_Suicide(char *s);
 void Rgnome_ShowMessage(char *s);
 int  Rgnome_ReadConsole(char *prompt, unsigned char *buf, int len, 
@@ -37,7 +35,16 @@ int  Rgnome_ShowFiles(int nfile, char **file, char **headers, char *wtitle,
 		      Rboolean del, char *pager);
 int  Rgnome_ChooseFile(int new, char *buf, int len);
 
+void Raqua_StartConsole(void);
+int  Raqua_ReadConsole(char *prompt, unsigned char *buf, int len, 
+			int addtohistory);
+void Raqua_WriteConsole(char *buf, int len);
+void Raqua_ResetConsole(void);
+void Raqua_FlushConsole(void);
+void Raqua_ClearerrConsole(void);
+
 void R_load_X11_shlib(void);
 void R_load_gnome_shlib(void);
+void R_load_aqua_shlib(void);
 
 int Rf_initEmbeddedR(int argc, char **argv);
